==> Botania-1.19.2-438-FABRIC-20230214.034242-11-sources.jar: vazkii/botania/client/render/world/BoundBlockRenderer.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.client.render.world;

import com.mojang.blaze3d.systems.RenderSystem;
import vazkii.botania.api.BotaniaAPI;
import vazkii.botania.api.item.CoordBoundItem;
import vazkii.botania.api.item.WireframeCoordinateListProvider;
import vazkii.botania.client.core.handler.ClientTickHandler;
import vazkii.botania.client.core.helper.RenderHelper;
import vazkii.botania.mixin.client.LevelRendererAccessor;
import vazkii.botania.xplat.BotaniaConfig;
import vazkii.botania.xplat.XplatAbstractions;

import java.util.IdentityHashMap;
import java.util.List;
import java.util.Map;
import net.minecraft.class_1263;
import net.minecraft.class_156;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1921;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_265;
import net.minecraft.class_287;
import net.minecraft.class_289;
import net.minecraft.class_310;
import net.minecraft.class_3532;
import net.minecraft.class_4184;
import net.minecraft.class_4587;
import net.minecraft.class_4588;
import net.minecraft.class_4597;

public final class BoundBlockRenderer {
	private static final class_4597.class_4598 LINE_BUFFERS = class_4597.method_22992(class_156.method_656(() -> {
		Map<class_1921, class_287> ret = new IdentityHashMap<>();
		ret.put(RenderHelper.LINE_1_NO_DEPTH, new class_287(RenderHelper.LINE_1_NO_DEPTH.method_22722()));
		ret.put(RenderHelper.LINE_4_NO_DEPTH, new class_287(RenderHelper.LINE_4_NO_DEPTH.method_22722()));
		ret.put(RenderHelper.LINE_5_NO_DEPTH, new class_287(RenderHelper.LINE_5_NO_DEPTH.method_22722()));
		ret.put(RenderHelper.LINE_8_NO_DEPTH, new class_287(RenderHelper.LINE_8_NO_DEPTH.method_22722()));
		return ret;
	}), class_289.method_1348().method_1349());

	private BoundBlockRenderer() {}

	public static void onWorldRenderLast(class_4184 camera, class_4587 ms, class_1937 level) {
		class_1657 player = class_310.method_1551().field_1724;
		if (!BotaniaConfig.client().boundBlockWireframe()
				|| player == null
				|| player.field_6002 != level) {
			return;
		}

		ms.method_22903();

		int color = 0xFF000000 | class_3532.method_15369(ClientTickHandler.ticksInGame % 200 / 200F, 0.6F, 1F);

		if (!player.method_6047().method_7960()) {
			var coordBoundItem = XplatAbstractions.INSTANCE.findCoordBoundItem(player.method_6047());
			if (coordBoundItem != null) {
				class_2338 coords = coordBoundItem.getBinding(player.field_6002);
				if (coords != null) {
					renderBlockOutlineAt(camera, ms, LINE_BUFFERS, player.field_6002, coords, color);
				}
			}
		}

		if (!player.method_6079().method_7960()) {
			var coordBoundItem = XplatAbstractions.INSTANCE.findCoordBoundItem(player.method_6079());
			if (coordBoundItem != null) {
				class_2338 coords = coordBoundItem.getBinding(player.field_6002);
				if (coords != null) {
					renderBlockOutlineAt(camera, ms, LINE_BUFFERS, player.field_6002, coords, color);
				}
			}
		}

		renderWireframeProviders(camera, player.method_31548(), player, ms, color);
		renderWireframeProviders(camera, BotaniaAPI.instance().getAccessoriesInventory(player), player, ms, color);

		ms.method_22909();
		RenderSystem.disableDepthTest();
		LINE_BUFFERS.method_22993();
	}

	private static void renderWireframeProviders(class_4184 camera, class_1263 inv,
			class_1657 player, class_4587 ms, int color) {
		for (int i = 0; i < inv.method_5439(); i++) {
			class_1799 stackInSlot = inv.method_5438(i);

			if (!stackInSlot.method_7960() && stackInSlot.method_7909() instanceof WireframeCoordinateListProvider provider) {
				List<class_2338> coordsList = provider.getWireframesToDraw(player, stackInSlot);
				for (class_2338 coords : coordsList) {
					renderBlockOutlineAt(camera, ms, LINE_BUFFERS, player.field_6002, coords, color);
				}

				class_2338 coords = provider.getSourceWireframe(player, stackInSlot);
				if (coords != null && coords.method_10264() != Integer.MIN_VALUE) {
					renderBlockOutlineAt(camera, ms, LINE_BUFFERS, player.field_6002, coords, color, true);
				}
			}
		}
	}

	private static void renderBlockOutlineAt(class_4184 camera, class_4587 ms, class_4597 buffers, class_1937 level, class_2338 pos, int color) {
		renderBlockOutlineAt(camera, ms, buffers, level, pos, color, false);
	}

	private static void renderBlockOutlineAt(class_4184 camera, class_4587 ms, class_4597 buffers, class_1937 level, class_2338 pos, int color, boolean thick) {
		class_265 shape = level.method_8320(pos).method_26218(level, pos);

		if (!shape.method_1110()) {
			double renderPosX = camera.method_19326().method_10216();
			double renderPosY = camera.method_19326().method_10214();
			double renderPosZ = camera.method_19326().method_10215();

			ms.method_22903();
			ms.method_22904(pos.method_10263() - renderPosX, pos.method_10264() - renderPosY, pos.method_10260() - renderPosZ);

			class_4588 buffer = buffers.getBuffer(thick ? RenderHelper.LINE_5_NO_DEPTH : RenderHelper.LINE_1_NO_DEPTH);
			renderBlockOutline(ms, buffer, shape, color);

			buffer = buffers.getBuffer(thick ? RenderHelper.LINE_8_NO_DEPTH : RenderHelper.LINE_4_NO_DEPTH);
			int alpha = 64;
			color = (color & ~0xff000000) | (alpha << 24);
			renderBlockOutline(ms, buffer, shape, color);

			ms.method_22909();
		}
	}

	private static void renderBlockOutline(class_4587 pose, class_4588 buffer, class_265 shape, int color) {
		float a = ((color >> 24) & 0xFF) / 255.0F;
		float r = ((color >> 16) & 0xFF) / 255.0F;
		float g = ((color >> 8) & 0xFF) / 255.0F;
		float b = (color & 0xFF) / 255.F;
		LevelRendererAccessor.renderShape(pose, buffer, shape, 0.0, 0.0, 0.0, r, g, b, a);
	}
}

==> Botania-1.20.1-451-FABRIC-20251018.024823-16-sources.jar: vazkii/botania/client/render/world/BoundBlockRenderer.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.client.render.world;

import com.mojang.blaze3d.systems.RenderSystem;
import vazkii.botania.api.BotaniaAPI;
import vazkii.botania.api.item.CoordBoundItem;
import vazkii.botania.api.item.WireframeCoordinateListProvider;
import vazkii.botania.client.core.handler.ClientTickHandler;
import vazkii.botania.client.core.helper.RenderHelper;
import vazkii.botania.mixin.client.LevelRendererAccessor;
import vazkii.botania.xplat.BotaniaConfig;
import vazkii.botania.xplat.XplatAbstractions;

import java.util.IdentityHashMap;
import java.util.List;
import java.util.Map;
import net.minecraft.class_1263;
import net.minecraft.class_156;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1921;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_265;
import net.minecraft.class_287;
import net.minecraft.class_289;
import net.minecraft.class_310;
import net.minecraft.class_3532;
import net.minecraft.class_4184;
import net.minecraft.class_4587;
import net.minecraft.class_4588;
import net.minecraft.class_4597;

public final class BoundBlockRenderer {
	private static final class_4597.class_4598 LINE_BUFFERS = class_4597.method_22992(class_156.method_656(() -> {
		Map<class_1921, class_287> ret = new IdentityHashMap<>();
		ret.put(RenderHelper.LINE_1_NO_DEPTH, new class_287(RenderHelper.LINE_1_NO_DEPTH.method_22722()));
		ret.put(RenderHelper.LINE_4_NO_DEPTH, new class_287(RenderHelper.LINE_4_NO_DEPTH.method_22722()));
		ret.put(RenderHelper.LINE_5_NO_DEPTH, new class_287(RenderHelper.LINE_5_NO_DEPTH.method_22722()));
		ret.put(RenderHelper.LINE_8_NO_DEPTH, new class_287(RenderHelper.LINE_8_NO_DEPTH.method_22722()));
		return ret;
	}), class_289.method_1348().method_1349());

	private BoundBlockRenderer() {}

	public static void onWorldRenderLast(class_4184 camera, class_4587 ms, class_1937 level) {
		class_1657 player = class_310.method_1551().field_1724;
		if (!BotaniaConfig.client().boundBlockWireframe()
				|| player == null
				|| player.method_37908() != level) {
			return;
		}

		ms.method_22903();

		int color = 0xFF000000 | class_3532.method_15369(ClientTickHandler.ticksInGame % 200 / 200F, 0.6F, 1F);

		if (!player.method_6047().method_7960()) {
			var coordBoundItem = XplatAbstractions.INSTANCE.findCoordBoundItem(player.method_6047());
			if (coordBoundItem != null) {
				class_2338 coords = coordBoundItem.getBinding(player.method_37908());
				if (coords != null) {
					renderBlockOutlineAt(camera, ms, LINE_BUFFERS, player.method_37908(), coords, color);
				}
			}
		}

		if (!player.method_6079().method_7960()) {
			var coordBoundItem = XplatAbstractions.INSTANCE.findCoordBoundItem(player.method_6079());
			if (coordBoundItem != null) {
				class_2338 coords = coordBoundItem.getBinding(player.method_37908());
				if (coords != null) {
					renderBlockOutlineAt(camera, ms, LINE_BUFFERS, player.method_37908(), coords, color);
				}
			}
		}

		renderWireframeProviders(camera, player.method_31548(), player, ms, color);
		renderWireframeProviders(camera, BotaniaAPI.instance().getAccessoriesInventory(player), player, ms, color);

		ms.method_22909();
		RenderSystem.disableDepthTest();
		LINE_BUFFERS.method_22993();
	}

	private static void renderWireframeProviders(class_4184 camera, class_1263 inv,
			class_1657 player, class_4587 ms, int color) {
		for (int i = 0; i < inv.method_5439(); i++) {
			class_1799 stackInSlot = inv.method_5438(i);

			if (!stackInSlot.method_7960() && stackInSlot.method_7909() instanceof WireframeCoordinateListProvider provider) {
				List<class_2338> coordsList = provider.getWireframesToDraw(player, stackInSlot);
				for (class_2338 coords : coordsList) {
					renderBlockOutlineAt(camera, ms, LINE_BUFFERS, player.method_37908(), coords, color);
				}

				class_2338 coords = provider.getSourceWireframe(player, stackInSlot);
				if (coords != null && coords.method_10264() != Integer.MIN_VALUE) {
					renderBlockOutlineAt(camera, ms, LINE_BUFFERS, player.method_37908(), coords, color, true);
				}
			}
		}
	}

	private static void renderBlockOutlineAt(class_4184 camera, class_4587 ms, class_4597 buffers, class_1937 level, class_2338 pos, int color) {
		renderBlockOutlineAt(camera, ms, buffers, level, pos, color, false);
	}

	private static void renderBlockOutlineAt(class_4184 camera, class_4587 ms, class_4597 buffers, class_1937 level, class_2338 pos, int color, boolean thick) {
		class_265 shape = level.method_8320(pos).method_26218(level, pos);

		if (!shape.method_1110()) {
			double renderPosX = camera.method_19326().method_10216();
			double renderPosY = camera.method_19326().method_10214();
			double renderPosZ = camera.method_19326().method_10215();

			ms.method_22903();
			ms.method_22904(pos.method_10263() - renderPosX, pos.method_10264() - renderPosY, pos.method_10260() - renderPosZ);

			class_4588 buffer = buffers.getBuffer(thick ? RenderHelper.LINE_5_NO_DEPTH : RenderHelper.LINE_1_NO_DEPTH);
			renderBlockOutline(ms, buffer, shape, color);

			buffer = buffers.getBuffer(thick ? RenderHelper.LINE_8_NO_DEPTH : RenderHelper.LINE_4_NO_DEPTH);
			int alpha = 64;
			color = (color & ~0xff000000) | (alpha << 24);
			renderBlockOutline(ms, buffer, shape, color);

			ms.method_22909();
		}
	}

	private static void renderBlockOutline(class_4587 pose, class_4588 buffer, class_265 shape, int color) {
		float a = ((color >> 24) & 0xFF) / 255.0F;
		float r = ((color >> 16) & 0xFF) / 255.0F;
		float g = ((color >> 8) & 0xFF) / 255.0F;
		float b = (color & 0xFF) / 255.F;
		LevelRendererAccessor.renderShape(pose, buffer, shape, 0.0, 0.0, 0.0, r, g, b, a);
	}
}
